@@ -53,7 +53,7 @@ def apply_unique(
 
     """
     if progress:
-        tqdm.pandas(desc='function evaluation', dynamic_ncols=True)
+        tqdm.pandas(desc='model evaluation', dynamic_ncols=True)
         applied_df = df.merge(df.drop_duplicates()
                               .assign(**{func.__name__: lambda x: x.progress_apply(func, axis=axis)}),
                               how='left')
@@ -749,7 +749,6 @@ def bootstrapping(
     cov_section_all: pd.DataFrame,
     bootstrap_size: int,
     bootstrap_ci: float,
-    func: Callable,
     delta_h: float,
     ivars_scales: Tuple[float, ...],
     parameters: Dict[Union[str, int], Tuple[float, float]],
@@ -760,8 +759,8 @@ def bootstrapping(
     progress: bool = False
 ) -> Tuple:
     """
-    performs bootstrapping procedure to gather confidence interval limits on the variogram, Sobol, and IVARS results,
-    and the reliability estimates of the variogram, Sobol, and IVARS resuls. Also groups the Sobol and IVARS50 results
+    performs bootstrapping procedure to gather confidence interval limits on the variogram, Sobol, IVARS  and VARS-ABE results,
+    and the reliability estimates of the variogram, Sobol, IVARS, and VARS-ABE results. Also groups the Sobol and IVARS50 results
     using clustering analysis in a hierarchical order
 
     Parameters
@@ -778,8 +777,6 @@ def bootstrapping(
         the number of bootstrap samples that were taken
     bootstrap_ci : float
         the confidence interval of the bootstrapping results (ex. 0.90)
-    func : callable
-        the function that performs the models calculations
     delta_h : float
         resolution of star samples
     ivars_scales : tuple
@@ -840,10 +837,16 @@ def bootstrapping(
     result_bs_variogram = pd.DataFrame()
     result_bs_sobol = pd.DataFrame()
     result_bs_ivars_df = pd.DataFrame()
+    result_bs_maee = pd.DataFrame()
     result_bs_sobol_ranking = pd.DataFrame()
     result_bs_ivars_ranking = pd.DataFrame()
 
-    for i in tqdm(range(0, bootstrap_size), desc='bootstrapping', disable=not progress, dynamic_ncols=True):
+    if grouping_flag:
+        bs = tqdm(range(0, bootstrap_size), desc='bootstrapping and grouping', disable=not progress, dynamic_ncols=True)
+    else:
+        bs = tqdm(range(0, bootstrap_size), desc='bootstrapping', disable=not progress, dynamic_ncols=True)
+
+    for i in bs:
         # bootstrapping to get CIs
         # specify random sequence by sampling with replacement
         bootstrap_rand = np.random.choice(
@@ -857,16 +860,18 @@ def bootstrapping(
         bootstrapped_cov_section_all = pd.concat(
             [cov_section_all.loc[pd.IndexSlice[i, :]] for i in bootstrap_rand])
 
-        # calculating variogram, ecovariogram, variance, mean, Sobol, and IVARS values
+        # calculating variogram, ecovariogram, variance, mean, Sobol, IVARS, and VARS-ABE values
         bootstrapped_variogram = variogram(bootstrapped_pairdf)
 
         bootstrapped_ecovariogram = e_covariogram(bootstrapped_cov_section_all)
 
-        bootstrapped_var = bootstrapped_df[func.__name__].unique().var(ddof=1)
+        bootstrapped_var = bootstrapped_df.iloc[:, -1].unique().var(ddof=1)
 
         bootstrapped_sobol = sobol_eq(bootstrapped_variogram, bootstrapped_ecovariogram,
                                       bootstrapped_var, delta_h)
 
+        bootstrapped_maee = morris_eq(bootstrapped_pairdf)[0]
+
         bootstrapped_ivars_df = pd.DataFrame.from_dict(
             {scale: bootstrapped_variogram.groupby(level=0).apply(ivars, scale=scale,
                                                                   delta_h=delta_h)
@@ -889,6 +894,9 @@ def bootstrapping(
         # unstack variogram so that results concat nicely
         bootstrapped_variogram_df = bootstrapped_variogram.unstack(level=0)
 
+        # unstack vars-abe so that results concat nicely
+        bootstrapped_maee_df = bootstrapped_maee.unstack(level=0)
+
         # swap sobol results rows and columns so that results concat nicely
         bootstrapped_sobol_df = bootstrapped_sobol.to_frame().transpose()
 
@@ -898,6 +906,7 @@ def bootstrapping(
         result_bs_sobol = pd.concat([bootstrapped_sobol_df, result_bs_sobol])
         result_bs_ivars_df = pd.concat(
             [bootstrapped_ivars_df, result_bs_ivars_df])
+        result_bs_maee = pd.concat([bootstrapped_maee_df, result_bs_maee])
         result_bs_sobol_ranking = pd.concat(
             [bootstrapped_sobol_ranking_df, result_bs_sobol_ranking])
         result_bs_ivars_ranking = pd.concat(
@@ -925,6 +934,29 @@ def bootstrapping(
     gammalb = gammalb.transpose()
     gammaub = gammaub.transpose()
 
+    # calculate upper and lower confidence interval limits for variogram results
+    maeelb = pd.DataFrame()
+    maeeub = pd.DataFrame()
+    # iterate through each h value
+    for h in np.unique(result_bs_maee.index.values).tolist():
+        # find all confidence interval limits for each h value
+        maeelb = pd.concat(
+            [maeelb,
+             result_bs_maee.loc[h].quantile((1 - bootstrap_ci) / 2).rename(h).to_frame()], axis=1)
+        maeeub = pd.concat(
+            [maeeub,
+             result_bs_maee.loc[h].quantile(1 - ((1 - bootstrap_ci) / 2)).rename(h).to_frame()],
+            axis=1)
+
+    # index value name is h?? not sure if this should be changed later
+    maeelb.index.names = ['h']
+    maeeub.index.names = ['h']
+
+    # transpose to get into correct format
+    maeelb = maeelb.transpose()
+    maeeub = maeeub.transpose()
+
+
     # calculate upper and lower confidence interval limits for sobol results in a nice looking format
     stlb = result_bs_sobol.quantile(
         (1 - bootstrap_ci) / 2).rename('').to_frame().transpose()
@@ -982,9 +1014,9 @@ def bootstrapping(
             grouping(result_bs_ivars_df, result_bs_sobol, result_bs_ivars_ranking, result_bs_sobol_ranking,
                      num_grps, st_factor_ranking, ivars_factor_ranking, parameters, bootstrap_size)
 
-        return gammalb, gammaub, stlb, stub, ivarslb, ivarsub, rel_sobol_factor_ranking,\
+        return gammalb, gammaub, maeelb, maeeub, stlb, stub, ivarslb, ivarsub, rel_sobol_factor_ranking,\
             rel_ivars_factor_ranking, ivars50_grp, sobol_grp, reli_sobol_grp, reli_ivars50_grp
     # if grouping is not chosen to be done return only bootstrapping results
     else:
-        return gammalb, gammaub, stlb, stub, ivarslb, ivarsub, rel_sobol_factor_ranking, \
+        return gammalb, gammaub, maeelb, maeeub, stlb, stub, ivarslb, ivarsub, rel_sobol_factor_ranking, \
             rel_ivars_factor_ranking
